/* Developed for Lenovo Educational Board Pass 2
   © Lenovo Created by Joshua Grosser
   v. 1.0 */

// include the following libraries
#include "AdafruitIO_WiFi.h"
#include <Adafruit_NeoPixel.h>
#include <Preferences.h>
Preferences preferences;

#define IO_USERNAME "*****" // input adafruit username here
#define IO_KEY "*****" // input adafruit key here
String ssid = "";
String password = "";
#define NEOPIXEL_PIN 45 // Neopixel is on pin 45
#define NEOPIXEL_COUNT 1 // There is only 1 RGB LED on DOS
#define NEOPIXEL_INDEX 0 // RGB LED is at index 0

AdafruitIO_WiFi io(IO_USERNAME, IO_KEY, ssid.c_str(), password.c_str());
Adafruit_NeoPixel Pixel(NEOPIXEL_COUNT, NEOPIXEL_PIN, NEO_GRB + NEO_KHZ800);

// set up feeds
AdafruitIO_Feed *sunrise_feed = io.feed("sunrise");
AdafruitIO_Feed *sunset_feed = io.feed("sunset");
AdafruitIO_Feed *rain_feed = io.feed("rain");
AdafruitIO_Feed *clearday_feed = io.feed("clearday");

const int yellow = 11;
const int blue = 13;
int c = 0;
int i = 0;
int j = 0;
int k = 0;

// get Wifi info
void getWifi()
{
  preferences.begin("credentials", false);
  ssid = preferences.getString("ssid", ""); 
  password = preferences.getString("password", "");
  if (ssid == "" || password == ""){
    Serial.println("No ssid or password saved.");
  }
  else {
    Serial.println("Successfully read WiFi information.");
  }
}

void setup() {
  // begin serial
  Serial.begin(115200); 
  getWifi();
  // set up LEDs and Neopixel
  pinMode(yellow, OUTPUT);
  pinMode(blue, OUTPUT);
  Pixel.begin();
  Pixel.clear();
  Pixel.show();
  
  // connect to adafruit
  Serial.print("Connecting to Adafruit IO");
  io.connect();
  
  // sets up function to be called when a message is recieved
  sunrise_feed->onMessage(sunrise);
  sunset_feed->onMessage(sunset);
  rain_feed->onMessage(rain);
  clearday_feed->onMessage(clearday);
  while(io.status() < AIO_CONNECTED) {
    Serial.print(".");
    delay(500);
  }
  Serial.println();
  Serial.println(io.statusText());
  sunrise_feed->get();
  sunset_feed->get();
  rain_feed->get();
  clearday_feed->get();
}

void loop() {
  // processes data
  io.run();
}

// called when sunrise recieved
void sunrise(AdafruitIO_Data *data) {
  if (c != 0)
  {
    Serial.println("Sunrise has been triggered!");
    // Turn Neopixel off
    Pixel.setPixelColor(NEOPIXEL_INDEX, 0, 0, 0);
    Pixel.show();
  } 
  c++;
}

// called when sunset recieved
void sunset(AdafruitIO_Data *data) {
  if (i != 0)
  {
    Serial.println("Sunset has been triggered!");
    // Turn Neopixel on
    Pixel.setPixelColor(NEOPIXEL_INDEX, 255, 255, 255);
    Pixel.show();
  } 
  i++;
}

// called when rain recieved
void rain(AdafruitIO_Data *data) {  
  if (j != 0)
  {
    Serial.println("Rain has been triggered!");
    // yellow LED off and blue LED on
    digitalWrite(yellow, LOW);
    digitalWrite(blue, HIGH);
  } 
  j++;
}

// called when clearday recieved
void clearday(AdafruitIO_Data *data) {  
  if (k != 0)
  {
    Serial.println("Clearday has been triggered!");
    // blue LED off and yellow LED on
    digitalWrite(blue, LOW);
    digitalWrite(yellow, HIGH);
  } 
  k++;
}
